
from async_unshortenit.module import AsyncUnshortenModule

class Redirect(AsyncUnshortenModule):

    name = "redirect"
    domains = set(["fb.me"])

    def __init__(self, headers: dict = None, timeout: int = 30):
        super().__init__(headers, timeout)

    async def unshorten(self, uri: str) -> str:
        res_text, url = await self.get(uri)
        return url



from .__version__ import (          # noqa
    __title__,
    __description__,
    __url__,
    __version__,
    __author__,
    __author_email__,
    __license__,
    __copyright__
)

from .async_unshortenit import AsyncUnshortenIt    # noqa



from lxml import html
from urllib.parse import urlparse
import re
import aiohttp
from async_unshortenit.module import AsyncUnshortenModule
from async_unshortenit.exceptions import AsyncUnshortenFailed
import asyncio


class LinkBucks(AsyncUnshortenModule):

    name = "linkbucks"
    domains = set(
        [
            "linkbucks.com",
            "www.linkbucks.com",
            "www.jzrputtbut.net",
            "any.gs",
            "cash4links.co",
            "cache4files.co",
            "dyo.gs",
            "filesonthe.net",
            "goneviral.com",
            "megaline.co",
            "miniurls.co",
            "qqc.co",
            "seriousdeals.net",
            "theseblogs.com",
            "theseforums.com",
            "tinylinks.co",
            "tubeviral.com",
            "ultrafiles.net",
            "urlbeat.net",
            "whackyvidz.com",
            "yyv.co",
        ]
    )

    def __init__(self, headers: dict = None, timeout: int = 30):
        super().__init__(headers, timeout)

    async def unshorten(self, uri: str) -> str:
        session = aiohttp.ClientSession()

        res = await session.get(
            uri, headers=self.headers, timeout=self.timeout, allow_redirects=True
        )

        html_tree = html.fromstring(await res.text())
        script_tags = html_tree.xpath("//script[contains(.,'var params')]")

        if len(script_tags) == 0:
            raise AsyncUnshortenFailed("Could not find decoder script.")

        lines = script_tags[-1].text_content().split("\n")

        numbers = []
        token = None
        keyUrl = None

        for line in lines:
            line = line.strip()
            if re.match(r"^params\['A.*y'\] \=.*;$", line):
                numbers.append(int(re.findall(r"[0-9]+", line)[0]))
            if re.match(r"^Token\:.*$", line):
                token = re.findall(r"'[0-9a-zA-Z]+'", line)[0].replace("'", "")
            if re.match(r"^KeyUrl\:.*$", line):
                keyUrl = re.findall("'.*'", line)[0].replace("'", "")

        if len(numbers) == 0 or token is None or keyUrl is None:
            raise AsyncUnshortenFailed("Failed to extract token and authkey.")

        authKey = sum(set(numbers))
        print(set(numbers), authKey, token, keyUrl)

        keyUrl = "http://{domain}{keyUrl}".format(
            domain=urlparse(res.url).netloc, keyUrl=keyUrl
        )
        print(keyUrl)

        headers = {
            "Accept": "/",
            "Accept-Encoding": "gzip, deflate",
            "Accept-Language": "en-US,en;q=0.5",
            "Connection": "keep-alive",
            "DNT": "1",
            "Host": urlparse(res.url).netloc,
            "Referrer": str(res.url),
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:58.0) Gecko/20100101 Firefox/58.0",  # noqa
        }

        await session.get(
            "http://www.linkbucks.com/scripts/intermissionLink.v13.js",
            headers=headers,
            timeout=self.timeout,
            allow_redirects=True,
        )
        await session.get(
            keyUrl, headers=headers, timeout=self.timeout, allow_redirects=True
        )

        params = dict(a_b=True, aK=authKey, t=token)

        await asyncio.sleep(5)
        intermission_url = "http://{domain}/intermission/loadTargetUrl".format(
            domain=urlparse(res.url).netloc
        )
        res = await session.get(
            intermission_url, params=params, headers=headers, allow_redirects=True
        )
        print(await res.text())

        await session.close()
        # ak = re.findall(r"params\['A.*y'] \= .*;", script)
        # for k in ak:
        #     print(k)



from typing import List
import aiohttp
from .module import AsyncUnshortenModule
from .modules import AdfLy, AdFocus, ShorteSt, MetaRefresh
from async_unshortenit import __version__


DEFAULT_HEADERS = {
    "User-Agent": "unshortenit {}".format(__version__),
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
    "Accept-Encoding": "gzip, deflate, br",
    "Accept-Language": "en-US,en;q=0.5",
    "Cache-Control": "no-cache",
}


class AsyncUnshortenIt:

    modules = {}
    _default_headers = None
    _default_timeout = None

    def __init__(self, default_timeout: int = 30, default_headers: dict = None):
        self._default_headers = default_headers or DEFAULT_HEADERS
        self._default_timeout = default_timeout

        self.register_modules([AdfLy, AdFocus, ShorteSt, MetaRefresh])

    def register_module(self, module: AsyncUnshortenModule):
        if not isinstance(module, AsyncUnshortenModule):
            module = module(
                headers=self._default_headers, timeout=self._default_timeout
            )
        self.modules[module.name] = module

    def register_modules(self, modules: List[AsyncUnshortenModule]):
        for module in modules:
            self.register_module(module)

    async def unshorten(
        self,
        uri: str,
        module: str = None,
        timeout: int = None,
        unshorten_nested: bool = False,
        force: bool = False,
    ) -> str:

        timeout = timeout or self._default_timeout

        if module and module in self.modules:
            return await self.modules[module].unshorten(uri)

        if unshorten_nested:
            last_uri = uri
            while True:
                matched = False
                for k, m in self.modules.items():
                    if m.is_match(uri):
                        matched = True
                        uri = await m.unshorten(uri)
                        if uri == last_uri:
                            break
                        last_uri = uri
                if not matched:
                    break
        else:
            for k, m in self.modules.items():
                if m.is_match(uri):
                    return await m.unshorten(uri)
        async with aiohttp.ClientSession() as session:
            async with session.get(
                url=uri,
                headers=self._default_headers,
                timeout=timeout,
                allow_redirects=True,
            ) as response:
                return str(response.url)



import re
from urllib.parse import urlsplit
import aiohttp
from async_unshortenit.exceptions import AsyncNotFound
from typing import Tuple


class AsyncUnshortenModule:

    name = None
    domains = set()
    _domain_regex = None

    def __init__(self, headers: dict = None, timeout: int = 30):
        self.headers = headers
        self.timeout = timeout
        self._build_domain_regex()

    def unshorten(self, uri: str, timeout: int = None) -> str:
        raise NotImplementedError

    def _build_domain_regex(self):
        regex_str = r"^(" + r"|".join(self.domains) + r")$"
        regex_str = re.sub(r"\.", "\.", regex_str)
        self._domain_regex = re.compile(regex_str)

    async def get(
        self, uri: str, headers: dict = None, timeout: int = None
    ) -> Tuple[str, str]:
        async with aiohttp.ClientSession() as session:
            async with session.get(
                url=uri,
                headers=headers or self.headers,
                timeout=timeout or self.timeout,
                allow_redirects=True,
            ) as response:
                if response.status != 200:
                    raise AsyncNotFound(response.status)
                return await response.text(), str(response.url)

    def is_match(self, url):
        domain = urlsplit(url).netloc
        return self._domain_regex.match(domain)

    def add_domain(self, domain):
        self.domains.add(domain)
        self._build_domain_regex()



import re
import copy
import asyncio
import json
import aiohttp


from async_unshortenit.module import AsyncUnshortenModule
from async_unshortenit.exceptions import AsyncUnshortenFailed


class ShorteSt(AsyncUnshortenModule):

    name = "shortest"
    domains = ["sh.st", "festyy.com", "ceesty.com"]

    def __init__(self, headers: dict = None, timeout: int = 30):
        super().__init__(headers, timeout)

    async def unshorten(self, uri: str) -> str:
        res_text, res_url = await self.get(uri)

        session_id = re.findall(r"sessionId\:(.*?)\"\,", res_text)
        if len(session_id) == 0:
            raise AsyncUnshortenFailed("No sessionId variable found.")

        if len(session_id) > 0:
            session_id = re.sub(r"\s\"", "", session_id[0])

            http_header = copy.copy(self.headers or {})
            http_header["Content-Type"] = "application/x-www-form-urlencoded"
            http_header["Host"] = "sh.st"
            http_header["Referer"] = uri
            http_header["Origin"] = "http://sh.st"
            http_header["X-Requested-With"] = "XMLHttpRequest"

            await asyncio.sleep(5)

            payload = {"adSessionId": session_id, "callback": "c"}
            async with aiohttp.ClientSession() as session:
                async with session.get(
                    "http://sh.st/shortest-url/end-adsession",
                    params=payload,
                    headers=http_header,
                    timeout=self.timeout,
                ) as r:
                    response = (await r.text())[6:-2]
                    if r.status == 200:
                        resp_uri = json.loads(response)["destinationUrl"]
                        if resp_uri is not None:
                            uri = resp_uri
                        else:
                            raise AsyncUnshortenFailed("Error extracting url.")
                    else:
                        raise AsyncUnshortenFailed("Error extracting url.")

        return uri



import re
import copy


from async_unshortenit.module import AsyncUnshortenModule
from async_unshortenit.exceptions import AsyncUnshortenFailed


class AdFocus(AsyncUnshortenModule):

    name = "adfocus"
    domains = set(["adfoc.us"])

    def __init__(self, headers: dict = None, timeout: int = 30):
        super().__init__(headers, timeout)

    async def unshorten(self, uri: str) -> str:
        orig_uri = uri
        res_text, url = await self.get(uri)

        adlink = re.findall("click_url =.*;", res_text)

        if len(adlink) == 0:
            raise AsyncUnshortenFailed("No click_uri variable found.")

        uri = re.sub('^click_url = "|"\;$', "", adlink[0])
        if re.search(r"http(s|)\://adfoc\.us/serve/skip/\?id\=", uri):
            http_header = copy.copy(self.headers)
            http_header["Host"] = "adfoc.us"
            http_header["Referrer"] = orig_uri

            res_text, url = await self.get(uri, headers=http_header)

            uri = url

        return uri



import re
from base64 import b64decode

from async_unshortenit.module import AsyncUnshortenModule
from async_unshortenit.exceptions import AsyncUnshortenFailed


class AdfLy(AsyncUnshortenModule):

    name = "adfly"
    domains = set(
        [
            "adf.ly",
            "j.gs",
            "q.gs",
            "u.bb",
            "ay.gy",
            "atominik.com",
            "tinyium.com",
            "microify.com",
        ]
    )

    def __init__(self, headers: dict = None, timeout: int = 30):
        super().__init__(headers=headers, timeout=timeout)

    async def unshorten(self, uri: str) -> str:
        """http://j.gs/AXr9"""
        res_text, url = await self.get(uri)

        ysmm = re.findall(r"var ysmm =.*\;?", res_text)

        if len(ysmm) == 0:
            raise AsyncUnshortenFailed("No ysmm variable found.")

        # Decode the ysmm variable and extract the actual link
        ysmm = re.sub(r"var ysmm \= \'|\'\;", "", ysmm[0])

        left = ""
        right = ""

        for c in [ysmm[i : i + 2] for i in range(0, len(ysmm), 2)]:
            left += c[0]
            right = c[1] + right

        # Additional digit arithmetic
        encoded_uri = list(left + right)
        numbers = ((i, n) for i, n in enumerate(encoded_uri) if str.isdigit(n))
        for first, second in zip(numbers, numbers):
            xor = int(first[1]) ^ int(second[1])
            if xor < 10:
                encoded_uri[first[0]] = str(xor)

        decoded_uri = b64decode("".join(encoded_uri).encode())[16:-16].decode()

        if re.search(r"go\.php\?u\=", decoded_uri):
            decoded_uri = b64decode(re.sub(r"(.*?)u=", "", decoded_uri)).decode()

        return decoded_uri



from lxml import html
from urllib.parse import urljoin



from async_unshortenit.module import AsyncUnshortenModule
from async_unshortenit.exceptions import AsyncUnshortenFailed


class MetaRefresh(AsyncUnshortenModule):

    name = "meta-refresh"
    domains = ["href.li", "anonymz.com"]

    def __init__(self, headers: dict = None, timeout: int = 30):
        super().__init__(headers, timeout)

    async def unshorten(self, uri: str) -> str:
        res_text, resp_url = await self.get(uri)

        html_tree = html.fromstring(res_text)
        meta_attr = html_tree.xpath(
            "//meta[translate(@http-equiv, 'REFSH', 'refsh') = 'refresh']/@content"
        )
        if not meta_attr:
            raise AsyncUnshortenFailed("No meta refresh tag present.")

        _, url = meta_attr[0].split(";")
        if url.strip().lower().startswith("url="):
            url = url[5:]
            if not url.startswith("http"):
                url = urljoin(resp_url, url)

            return url
        else:
            raise AsyncUnshortenFailed("Failed to extract meta refresh tag.")



__title__ = "async-unshortenit"
__description__ = "Unshortens ad-based shorteners and any 301 redirected urls."
__url__ = "https://github.com/kshubham506/async_unshortenit"
__version__ = "0.1.0"
__author__ = "Shubham Kumar"
__author_email__ = "kshubham506@gmail.com"
__license__ = "MIT"
__copyright__ = "Copyright 2018 Shubham Kumar"



from setuptools import setup, find_packages
import os


here = os.path.abspath(os.path.dirname(__file__))


about = {}
with open(os.path.join(here, 'async_unshortenit', '__version__.py'), 'r') as f:
    exec(f.read(), about)

setup(
    name=about['__title__'],
    version=about['__version__'],
    description=about['__description__'],
    author=about['__author__'],
    author_email=about['__author_email__'],
    url=about['__url__'],
    packages=find_packages(),
    package_data={'': ['LICENSE']},
    include_package_data=True,
    zip_safe=True,
    setup_requires=['pytest-runner<=3.0.1'],
    tests_require=['pytest'],
    install_requires=[
        'aiohttp>=3.8.1',
        'click>=8.1.3',
        'lxml>=4.9.1'
    ],
    license=about['__license__'],
    keywords='unshortener adf.ly lnx.lu sh.st shortener',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6'
    ]
)